==> fastbots-0.1.3.tar.gz: fastbots-0.1.3/fastbots/bot.py <==
import tempfile
import shutil
import pickle
from typing import List, Union
from pathlib import Path
from datetime import datetime
from configparser import ConfigParser
import logging
from typing import Type
from abc import ABC, abstractmethod

from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.remote.webdriver import WebDriver

from fastbots import config
from fastbots.exceptions import ExpectedUrlError


logger = logging.getLogger(__name__)


class Bot(ABC):
    """
    Bot

    Class used to specify a bot blueprint.
    """

    def __init__(self) -> None:
        """
        Bot

        Initialize all the attributes of the Bot instance
        """
        super().__init__()

        # use a temporary directory as default download folder
        if config.BOT_DOWNLOAD_FOLDER_PATH != 'None':
            self._temp_dir: str = tempfile.mkdtemp(dir=config.BOT_DOWNLOAD_FOLDER_PATH)
        else:
            self._temp_dir: str = tempfile.mkdtemp()

        # load all the locators
        self._locators: ConfigParser = self.__load_locators__()
        # data store
        self._payload: dict = {}

    @property
    def driver(self) -> WebDriver:
        """
        Driver Getter
        """
        return self._driver
    
    @property
    def wait(self) -> WebDriverWait:
        """
        Wait Getter
        """
        return self._wait
    
    @property
    def payload(self) -> dict:
        """
        Payload Getter
        """
        return self._payload

    def __enter__(self) -> Type['Bot']:
        """
        Enter

        Load and configure all the needed resources.
        """
        # default global driver settings
        self._driver.implicitly_wait(config.SELENIUM_GLOBAL_IMPLICIT_WAIT)

        # load the start page
        self._driver.get(self.locator('pages_url', 'start_url'))

        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        """
        Exit
        
        Clean all the used resources.
        """
        shutil.rmtree(self._temp_dir)
        self._driver.close()

    def check_page_url(self, expected_page_url: str):
        """
        Check Page Url

        Check that the browser it't at the expected page.
        """
        try:
            # polling that the page url is the expected
            WebDriverWait(driver=self._driver, timeout=config.SELENIUM_EXPECTED_URL_TIMEOUT, poll_frequency=1).until(
                EC.url_to_be(expected_page_url)
            )

        except TimeoutException as te:
            # if not the expected url raises an exception
            raise ExpectedUrlError(current_url=self._driver.current_url, expected_url=expected_page_url)

    def locator(self, page_name: str, locator_name: str) -> str:
        """
        Locator

        Getter that get the locator used for a page locator
        """
        if not self._locators.has_section(page_name):
            raise ValueError(f'The specified page_name: {page_name} is not declared in locators config.')
        if not self._locators.has_option(page_name, locator_name):
            raise ValueError(f'The specified locator_name: {locator_name} is not declared in locators config.')
        return self._locators.get(page_name, locator_name)
        
    def save_screenshot(self):
        """
        Save Screenshot

        Save the browser's screenshot to a png file, the path could be specified in the settings.
        """
        if not Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH).exists():
            Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH).mkdir(exist_ok=True, parents=True)

        file_path: Path = Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH) / f'{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}.png'
        self._driver.save_screenshot(str(file_path.absolute()))

    def save_html(self):
        """
        Save Html

        Save the browser's html page to a file, the path could be specified in the settings.
        """
        if not Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH).exists():
            Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH).mkdir(exist_ok=True, parents=True)

        file_path: Path = Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH) / f'{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}.html'
        with open(str(file_path.absolute()), "w", encoding="utf-8") as file:
            file.write(self._driver.page_source)

    def save_cookies(self):
        """
        Save Cookies

        Save all the cookies founded in the file.
        """
        cookies: List[dict] = self._driver.get_cookies()

        with open(config.BOT_COOKIES_FILE_PATH, 'wb') as file:
            pickle.dump(cookies, file)
    
    def load_cookies(self):
        """
        Load Cookies

        Add all the cookies founded in the file.
        """
        if Path(config.BOT_COOKIES_FILE_PATH).is_file():
            with open(config.BOT_COOKIES_FILE_PATH, 'rb') as file:
                cookies = pickle.load(file)

                for cookie in cookies:
                    self._driver.add_cookie(cookie)

    def __load_locators__(self) -> ConfigParser:
        """
        Load Locators

        Load a file that contains all the locators
        """
        if not Path(config.SELENIUM_LOCATORS_FILE).is_file():
            return ValueError(f'Erorr, locators file not founded at path: {config.SELENIUM_LOCATORS_FILE}')
        
        config_parser: ConfigParser = ConfigParser()
        config_parser.read(config.SELENIUM_LOCATORS_FILE)
        return config_parser

    @abstractmethod
    def __load_preferences__(self) -> Union[FirefoxProfile, dict]:
        """
        Load Preferences

        Load all the preferences stored in a json file,
        specified in the config.
        """
        return NotImplementedError('Bot must define this method.')

    @abstractmethod
    def __load_options__(self) -> Union[FirefoxOptions, ChromeOptions]:
        """
        Load Options

        Load all the default options
        """
        return NotImplementedError('Bot must define this method.')
    
    @abstractmethod
    def __load_driver__(self) -> WebDriver:
        """
        Load Driver

        Load and configure all the options for the driver.
        """
        return NotImplementedError('Bot must define this method.')

==> fastbots-0.1.5.tar.gz: fastbots-0.1.5/fastbots/bot.py <==
import tempfile
import shutil
import pickle
from typing import List, Union
from pathlib import Path
from datetime import datetime
from configparser import ConfigParser
import logging
from typing import Type
from abc import ABC, abstractmethod

from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.remote.webdriver import WebDriver

from fastbots import config, logger
from fastbots.exceptions import ExpectedUrlError, DownloadFileError


logger = logging.getLogger(__name__)


class Bot(ABC):
    """
    Bot

    Class used to specify a bot blueprint.
    """

    def __init__(self) -> None:
        """
        Bot

        Initialize all the attributes of the Bot instance
        """
        super().__init__()

        # use a temporary directory as default download folder
        self._temp_dir: str = tempfile.mkdtemp()

        # official downloaded file folder
        if config.BOT_DOWNLOAD_FOLDER_PATH != 'None':
            self._download_dir: str = tempfile.mkdtemp(dir=config.BOT_DOWNLOAD_FOLDER_PATH)
        else:
            self._download_dir: str = tempfile.mkdtemp()

        # load all the locators
        self._locators: ConfigParser = self.__load_locators__()
        # data store
        self._payload: dict = {}

    @property
    def driver(self) -> WebDriver:
        """
        Driver Getter
        """
        return self._driver
    
    @property
    def wait(self) -> WebDriverWait:
        """
        Wait Getter
        """
        return self._wait
    
    @property
    def payload(self) -> dict:
        """
        Payload Getter
        """
        return self._payload

    def __enter__(self) -> Type['Bot']:
        """
        Enter

        Load and configure all the needed resources.
        """
        # default global driver settings
        self._driver.implicitly_wait(config.SELENIUM_GLOBAL_IMPLICIT_WAIT)

        # load the start page
        self._driver.get(self.locator('pages_url', 'start_url'))

        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        """
        Exit
        
        Clean all the used resources.
        """
        shutil.rmtree(self._temp_dir)
        self._driver.close()

    def check_page_url(self, expected_page_url: str):
        """
        Check Page Url

        Check that the browser it't at the expected page.
        """
        try:
            # polling that the page url is the expected
            WebDriverWait(driver=self._driver, timeout=config.SELENIUM_EXPECTED_URL_TIMEOUT, poll_frequency=1).until(
                EC.url_to_be(expected_page_url)
            )

        except TimeoutException as te:
            # if not the expected url raises an exception
            raise ExpectedUrlError(current_url=self._driver.current_url, expected_url=expected_page_url)

    def locator(self, page_name: str, locator_name: str) -> str:
        """
        Locator

        Getter that get the locator used for a page locator
        """
        if not self._locators.has_section(page_name):
            raise ValueError(f'The specified page_name: {page_name} is not declared in locators config.')
        
        if not self._locators.has_option(page_name, locator_name):
            raise ValueError(f'The specified locator_name: {locator_name} is not declared in locators config.')
        
        return self._locators.get(page_name, locator_name)
        
    def wait_downloaded_file_path(self, file_extension: str, new_file_name: str | None = None) -> str:
        """
        Wait Downloaded File Path

        This method allow to wait for a specific downloaded file to be completely available in the download folder.
        It uses the file extension in order to wait the full download finish.
        It will also give the ability to rename the downloaded file.

        The file_extension must be specified without the dot "." (ex .png become png)
        """
        try:
            # polling that the page url is the expected, it uses the extension because the temp part file cache by browser
            # usally have a specific extension that isn't the usally of the files
            WebDriverWait(driver=self._driver, timeout=config.SELENIUM_FILE_DOWNLOAD_TIMEOUT, poll_frequency=1).until(
                lambda driver: len(list(Path(self._temp_dir).glob(f'*.{file_extension}'))) == 1
            )

            # get the latest downloaded file
            latest_file: Path = max(list(Path(self._temp_dir).glob(f'*.{file_extension}')), key=lambda x: x.stat().st_ctime)

            # build the download path based on renamed file or 
            downloaded_file_path: Path = None
            if new_file_name is None:
                downloaded_file_path = Path(config.BOT_DOWNLOAD_FOLDER_PATH) / latest_file.name
            else:
                downloaded_file_path = Path(config.BOT_DOWNLOAD_FOLDER_PATH) / f'{new_file_name}.{file_extension}'
                
            # move to the download folder the file name
            shutil.move(src=str(latest_file.absolute()), dst=str(downloaded_file_path.absolute()))

            # remove the temporary downloaded file
            latest_file.unlink()

            # return the path and filename as string
            return str(downloaded_file_path.absolute())

        except TimeoutException as te:
            # if not the expected url raises an exception
            file_count: int = len(list(Path(self._temp_dir).glob(f'*.{file_extension}')))

            # error string based on the specific error
            if file_count == 0:
                raise DownloadFileError('File not founded in the download folder, an error with the download occurs.')
            elif file_count > 1:
                raise DownloadFileError(f'Too many downloaded files founded, files number : {file_count}.')

            raise DownloadFileError()

    def save_screenshot(self):
        """
        Save Screenshot

        Save the browser's screenshot to a png file, the path could be specified in the settings.
        """
        if not Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH).exists():
            Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH).mkdir(exist_ok=True, parents=True)

        file_path: Path = Path(config.BOT_SCREENSHOT_DOWNLOAD_FOLDER_PATH) / f'{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}.png'
        self._driver.save_screenshot(str(file_path.absolute()))

    def save_html(self):
        """
        Save Html

        Save the browser's html page to a file, the path could be specified in the settings.
        """
        if not Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH).exists():
            Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH).mkdir(exist_ok=True, parents=True)

        file_path: Path = Path(config.BOT_HTML_DOWNLOAD_FOLDER_PATH) / f'{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}.html'
        with open(str(file_path.absolute()), "w", encoding="utf-8") as file:
            file.write(self._driver.page_source)

    def save_cookies(self):
        """
        Save Cookies

        Save all the cookies founded in the file.
        """
        cookies: List[dict] = self._driver.get_cookies()

        with open(config.BOT_COOKIES_FILE_PATH, 'wb') as file:
            pickle.dump(cookies, file)
    
    def load_cookies(self):
        """
        Load Cookies

        Add all the cookies founded in the file.
        """
        if Path(config.BOT_COOKIES_FILE_PATH).is_file():
            with open(config.BOT_COOKIES_FILE_PATH, 'rb') as file:
                cookies = pickle.load(file)

                for cookie in cookies:
                    self._driver.add_cookie(cookie)

    def __load_locators__(self) -> ConfigParser:
        """
        Load Locators

        Load a file that contains all the locators
        """
        if not Path(config.SELENIUM_LOCATORS_FILE).is_file():
            return ValueError(f'Erorr, locators file not founded at path: {config.SELENIUM_LOCATORS_FILE}')
        
        config_parser: ConfigParser = ConfigParser()
        config_parser.read(config.SELENIUM_LOCATORS_FILE)
        return config_parser

    @abstractmethod
    def __load_preferences__(self) -> Union[FirefoxProfile, dict]:
        """
        Load Preferences

        Load all the preferences stored in a json file,
        specified in the config.
        """
        return NotImplementedError('Bot must define this method.')

    @abstractmethod
    def __load_options__(self) -> Union[FirefoxOptions, ChromeOptions]:
        """
        Load Options

        Load all the default options
        """
        return NotImplementedError('Bot must define this method.')
    
    @abstractmethod
    def __load_driver__(self) -> WebDriver:
        """
        Load Driver

        Load and configure all the options for the driver.
        """
        return NotImplementedError('Bot must define this method.')
